;;;; match-action-support.scm
;;;; Kon Lovett, Jul '06

(use match #;srfi-12)

(eval-when (compile)
	(declare
		(usual-integrations)
		(fixnum)
		(inline)
		(import
			match-error-procedure
			match-error-control)
		(bound-to-procedure
			match-error-procedure
			match-error-control)
		(export
		  ma$toggle
			ma$prfm) ) )

;;;

(define-constant MATCHABLE-ERROR-MESSAGE "no matching pattern")

(define (make-match-exception val . rest)
	(make-composite-condition
		(apply make-property-condition 'exn
		  'location 'match-action
			'message MATCHABLE-ERROR-MESSAGE
			'arguments val rest)
		(make-property-condition 'match)) )

(define (match-error val . rest)
	(abort (apply make-match-exception val rest)) )

(define match-error-condition?
	(let ([exn-cnd? (condition-predicate 'exn)]
				[match-cnd? (condition-predicate 'match)]
				[exn-msg-prp (condition-property-accessor 'exn 'message)])
		(lambda (obj)
			(and (exn-cnd? obj)
			     (or (match-cnd? obj)
			         (equal? MATCHABLE-ERROR-MESSAGE (exn-msg-prp obj)) ) ) ) ) )

;;;

(define (ma$toggle . state)
	(if (pair? state)
		(and-let* ([new-state (car state)])
			(match-error-procedure (car new-state))
			(match-error-control (cdr new-state)))
		(let ([old-state (cons (match-error-procedure) (match-error-control))])
			(match-error-control #:error)
			(match-error-procedure match-error)
			old-state)) )

(define (ma$prfm ost mtc)
  (call/cc
    (lambda (k)
      (with-exception-handler
        (lambda (exp)
          (if (match-error-condition? exp)
            (k #f)
            (begin
              (ma$toggle ost)
              (abort exp))))
        (lambda ()
          (mtc)
          (ma$toggle ost)
          #t)))) )
